local VERSION = tonumber(("$Revision: 36839 $"):match("%d+"))

local PitBull = PitBull
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-05-25 03:42:00 -0400 (Fri, 25 May 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local _,playerClass = UnitClass("player")

local L = PitBull:L("PitBull-ABF")

local PitBull_Aura = nil

local DogTag = AceLibrary("DogTag-1.0")

local newFrame = PitBull.newFrame

local ABF = {}
PitBull.ABF = ABF

local ABF_db = PitBull:AcquireDBNamespace("Layout-ABF")
ABF.db = ABF_db
PitBull:RegisterDefaults("Layout-ABF", "profile", {
	["**"] = {
		portraitSide = "left",
		auraSide = "bottom",
		--note, these are only used when debuffs are split
		buffSide = "bottom",
		debuffSide = "right",
		
		classTextCustom = "[Classification] [Level:DifficultyColor] [SmartClass:ClassColor] [DruidForm:Paren] [SmartRace]",
		classTextStyle = "Standard",
		classTextPosition = "powerBar-left",
		classTextSize = 1,
		
		healthTextCustom = "[CurHP]/[MaxHP]",
		healthTextStyle = "Smart",
		healthTextPosition = "healthBar-right",
		healthTextSize = 1,
		
		nameTextCustom = "[Name]",
		nameTextStyle = "Standard",
		nameTextPosition = "healthBar-left",
		nameTextSize = 1,
		
		powerTextCustom = "[CurMP]/[MaxMP]",
		powerTextStyle = "Absolute",
		powerTextPosition = "powerBar-right",
		powerTextSize = 1,
		
		comboTextStyle = "Standard",
		comboTextPosition = "frame-right",
		comboTextSize = 1,
		
		experienceTextStyle = "Standard",
		experienceTextPosition = "expBar-center",
		experienceTextSize = 1,
		
		repTextStyle = "Standard",
		repTextPosition = "repBar-center",
		repTextSize = 1,
		
		combatTextPosition = "frame-center",
		
		healthBarHeight = 5,
		powerBarHeight = 4,
		castBarHeight = 2,
		repBarHeight = 2,
		expBarHeight = 2,
		druidManaBarHeight = 0.25,
		portraitHeight = 2,
	},
	player = {
		healthTextStyle = "Informational",
	},
	pet = {
		healthTextStyle = "Informational",
	},
	target = {
		portraitSide = "right",
		auraSide = "right",
		comboSide = "bottom",
		healthTextStyle = "Informational",
	}
})

local heights = {
	healthBar = 'healthBarHeight',
	powerBar = 'powerBarHeight',
	castBar = 'castBarHeight',
	repBar = 'repBarHeight',
	expBar = 'expBarHeight',
	portrait = 'portraitHeight',
}
ABF.heights = heights

local function figureHeight(multiplier, frame)
	local num = 0
	local db = ABF_db.profile[frame.group]
	for k,v in pairs(heights) do
		if frame[k] and frame[k]:IsShown() then
			if k == "portrait" then
				local portraitSide = ABF_db.profile[frame.group].portraitSide
				if portraitSide == "top" then
					num = num + db[v]
				end
			else
				num = num + db[v]
			end
		end
	end
	return db[heights[multiplier]] * frame:GetHeight() / num
end
ABF.figureHeight = figureHeight

function ABF.healthBar(unit, frame)
	local height = figureHeight('healthBar', frame)
	if frame.portrait then
		local portraitSide = ABF_db.profile[frame.group].portraitSide
		if portraitSide == "right" then
			frame.healthBar:SetPoint("TOP", frame, "TOP", 0, 0)
			frame.healthBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
			frame.healthBar:SetPoint("RIGHT", frame.portrait, "LEFT", -2, 0)
		elseif portraitSide == "left" then
			frame.healthBar:SetPoint("TOP", frame, "TOP", 0, 0)
			frame.healthBar:SetPoint("LEFT", frame.portrait, "RIGHT", 2, 0)
			frame.healthBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		else -- top
			frame.healthBar:SetPoint("TOP", frame.portrait, "BOTTOM", 0, 0)
			frame.healthBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
			frame.healthBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		end
	else
		frame.healthBar:SetPoint("TOP", frame, "TOP", 0, 0)
		frame.healthBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
		frame.healthBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	end
	frame.healthBar:SetHeight(height) -- no pixel spacing, there is always 1 spacing less than bars.
end
function ABF.powerBar(unit, frame)
	local height = figureHeight('powerBar', frame)
	if frame.healthBar then
		frame.powerBar:SetPoint("TOP", frame.healthBar, "BOTTOM", 0, -1)
	else
		frame.powerBar:SetPoint("TOP", frame, "TOP", 0, 0)
	end
	if frame.portrait then
		local portraitSide = ABF_db.profile[frame.group].portraitSide
		if portraitSide == "right" then
			frame.powerBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
			frame.powerBar:SetPoint("RIGHT", frame.portrait, "LEFT", -2, 0)
		elseif portraitSide == "left" then
			frame.powerBar:SetPoint("LEFT", frame.portrait, "RIGHT", 2, 0)
			frame.powerBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		else -- top
			frame.powerBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
			frame.powerBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		end
	else
		frame.powerBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
		frame.powerBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	end
	if frame.druidManaBar then
		height = height * (1 - ABF_db.profile[frame.group].druidManaBarHeight)
	end
	frame.powerBar:SetHeight(height - 1)
end
if playerClass == "DRUID" then
	function ABF.druidManaBar(unit, frame)
		if not frame.powerBar then
			return
		end
		local height = figureHeight('powerBar', frame) * ABF_db.profile[frame.group].druidManaBarHeight
		frame.druidManaBar:SetPoint("TOPLEFT", frame.powerBar, "BOTTOMLEFT")
		frame.druidManaBar:SetPoint("TOPRIGHT", frame.powerBar, "TOPRIGHT")
		frame.druidManaBar:SetHeight(height)
	end
end
function ABF.castBar(unit, frame)
	local height = figureHeight('castBar', frame)
	frame.castBar:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", height, 0)
	frame.castBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
	frame.castBar.bg:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
	frame.castBar.bg:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
	frame.castBar:SetHeight( height - 1) -- pixel spacing
	frame.castBar.bg:SetHeight(height - 1) -- pixel spacing
end
function ABF.repBar(unit, frame)
	local height = figureHeight('repBar', frame)
	frame.repBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
	frame.repBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	if frame.expBar then
		frame.repBar:SetPoint("BOTTOM", frame.expBar, "TOP", 0, 1) -- 1 up spacing
	elseif frame.castBar then
		frame.repBar:SetPoint("BOTTOM", frame.castBar, "TOP", 0, 1) -- 1 up spacing
	else	
		frame.repBar:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0)
	end	
	frame.repBar:SetHeight(height - 1)
end
function ABF.expBar(unit, frame)
	local height = figureHeight('expBar', frame)
	frame.expBar:SetPoint("LEFT", frame, "LEFT", 0, 0)
	frame.expBar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	if frame.castBar then
		frame.expBar:SetPoint("BOTTOM", frame.castBar, "TOP", 0, 1) -- 1 up spacing
	else
		frame.expBar:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0)
	end	
	frame.expBar:SetHeight(height - 1)
end
function ABF.castBarIcon(unit, frame)
	local height = figureHeight('castBar', frame)
	frame.castBarIcon:SetHeight( height -1)
	frame.castBarIcon:SetWidth( height -1)
	frame.castBarIcon:SetPoint("RIGHT", frame.castBar, "LEFT" )
	frame.castBarIcon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
end
function ABF.castBarText(unit, frame)
	frame.castBarText:ClearAllPoints()
	frame.castBarText:SetAllPoints(frame.castBar)
	frame.castBarText:SetJustifyH("LEFT")
end
function ABF.castBarTimeText(unit, frame)
	frame.castBarTimeText:ClearAllPoints()
	frame.castBarTimeText:SetAllPoints(frame.castBar)
	frame.castBarTimeText:SetJustifyH("RIGHT")
end

local function findFrameAtPosition(group, position)
	for k, v in pairs(ABF_db.profile[group]) do
		if v == position and type(k) == "string" and k:find("Position$") then
			return k:sub(1, -9)
		end
	end
end
local function textPosition(key)
	local key_position = key .. "Position"
	return function(unit, frame)
		local group = frame.group
		local position = ABF_db.profile[group][key_position] or ""
		local attach, point = ("-"):split(position)
		if attach and point then
			if attach == "frame" then
				if point == "center" then
					frame[key]:SetPoint("CENTER", frame, "CENTER", 0, 0)
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("MIDDLE")
				elseif point == "topleft" then
					frame[key]:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 2, 5)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("BOTTOM")
					local center = findFrameAtPosition(group, "frame-top")
					if frame[center] then
						frame[key]:SetPoint("RIGHT", frame[center], "LEFT", -3, 0)
					else
						local right = findFrameAtPosition(group, "frame-topright")
						if frame[right] then
							frame[key]:SetPoint("RIGHT", frame[right], "LEFT", -3, 0)
						end
					end
				elseif point == "topright" then
					frame[key]:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -2, 5)
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("BOTTOM")
					local center = findFrameAtPosition(group, "frame-top")
					if frame[center] then
						frame[key]:SetPoint("LEFT", frame[center], "RIGHT", 3, 0)
					end
				elseif point == "top" then
					frame[key]:SetPoint("BOTTOM", frame, "TOP", 0, 5)
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("BOTTOM")
				elseif point == "bottomleft" then
					frame[key]:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 2, -5)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("TOP")
					local center = findFrameAtPosition(group, "frame-bottom")
					if frame[center] then
						frame[key]:SetPoint("RIGHT", frame[center], "LEFT", -3, 0)
					else
						local right = findFrameAtPosition(group, "frame-bottomright")
						if frame[right] then
							frame[key]:SetPoint("RIGHT", frame[right], "LEFT", -3, 0)
						end
					end
				elseif point == "bottomright" then
					frame[key]:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", -2, -5)
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("TOP")
					local center = findFrameAtPosition(group, "frame-bottom")
					if frame[center] then
						frame[key]:SetPoint("LEFT", frame[center], "RIGHT", 3, 0)
					end
				elseif point == "bottom" then
					frame[key]:SetPoint("TOP", frame, "BOTTOM", 0, -5)
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("TOP")
				elseif point == "lefttop" then
					frame[key]:SetPoint("TOPRIGHT", frame, "TOPLEFT", -5, -2)
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("TOP")
				elseif point == "left" then
					frame[key]:SetPoint("RIGHT", frame, "LEFT", -5, 0)
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("MIDDLE")
				elseif point == "leftbottom" then
					frame[key]:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", -5, 2)
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("BOTTOM")
				elseif point == "righttop" then
					frame[key]:SetPoint("TOPLEFT", frame, "TOPRIGHT", 5, -2)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("TOP")
				elseif point == "right" then
					frame[key]:SetPoint("LEFT", frame, "RIGHT", 5, 0)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("MIDDLE")
				elseif point == "rightbottom" then
					frame[key]:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 5, 2)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("BOTTOM")
				end
			elseif frame[attach] then
				if point == "left" then
					frame[key]:SetPoint("LEFT", frame[attach], "LEFT", 3, 0)
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("MIDDLE")
					local center = findFrameAtPosition(group, attach .. "-center")
					if frame[center] then
						frame[key]:SetPoint("RIGHT", frame[center], "LEFT", -3, 0)
					else
						local right = findFrameAtPosition(group, attach .. "-right")
						if frame[right] then
							frame[key]:SetPoint("RIGHT", frame[right], "LEFT", -3, 0)
						end
					end
				elseif point == "center" then
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("MIDDLE")
					frame[key]:SetPoint("CENTER", frame[attach], "CENTER", 0, 0)
				elseif point == "right" then
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("MIDDLE")
					frame[key]:SetPoint("RIGHT", frame[attach], "RIGHT", -3, 0)
					local center = findFrameAtPosition(group, attach .. "-center")
					if frame[center] then
						frame[key]:SetPoint("LEFT", frame[center], "RIGHT", 3, 0)
					end
				elseif point == "top" then
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("TOP")
					frame[key]:SetPoint("TOP", frame[attach], "TOP", 0, -3)
				elseif point == "bottom" then
					frame[key]:SetJustifyH("CENTER")
					frame[key]:SetJustifyV("BOTTOM")
					frame[key]:SetPoint("BOTTOM", frame[attach], "BOTTOM", 0, 3)
				elseif point == "outright" then
					frame[key]:SetJustifyH("LEFT")
					frame[key]:SetJustifyV("MIDDLE")
					frame[key]:SetPoint("LEFT", frame[attach], "RIGHT", 3, 0)
				elseif point == "outleft" then
					frame[key]:SetJustifyH("RIGHT")
					frame[key]:SetJustifyV("MIDDLE")
					frame[key]:SetPoint("RIGHT", frame[attach], "LEFT", -3, 0)
				end
			end
		end
	end
end

ABF.experienceText = textPosition('experienceText')
ABF.repText = textPosition('repText')
ABF.healthText = textPosition('healthText')
ABF.powerText = textPosition('powerText')
ABF.nameText = textPosition('nameText')
ABF.classText = textPosition('classText')
if playerClass == "ROGUE" or playerClass == "DRUID" then
	ABF.comboText = textPosition('comboText')
end
ABF.combatText = textPosition('combatText')

function ABF.auraFrame(unit, frame)
	frame.auraFrame:ClearAllPoints()
	local size = PitBull:GetModule("Aura").db.profile[frame.group].buffsize
	frame.auraFrame:SetHeight(size)
	if PitBull:GetModule("Aura").db.profile[frame.group].split then
		local buffside = ABF_db.profile[frame.group].buffSide
		local w = frame:GetWidth()
		frame.auraFrame:SetWidth(w)
		if buffside == "bottom" then
			frame.auraFrame:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", -2, -2)
		elseif buffside == "top" then
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", -2, 2)
		elseif buffside == "left" then
			frame.auraFrame:SetPoint("BOTTOMRIGHT", frame, "LEFT", w-size-2, -2)
		elseif buffside == "right" then
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "RIGHT", 2, 0)
		elseif buffside == "topleft" then
			frame.auraFrame:SetPoint("TOPRIGHT", frame, "TOPLEFT", w-size-2, 0)
		elseif buffside == "topright" then
			frame.auraFrame:SetPoint("TOPLEFT", frame, "TOPRIGHT", 2, 0)
		elseif buffside == "bottomleft" then
			frame.auraFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", w-size-2, 0)
		elseif buffside == "bottomright" then
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 2, 0)
		end
	else
		local side = ABF_db.profile[frame.group].auraSide
		if side == "bottom" then
			frame.auraFrame:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", -2, -2)
			frame.auraFrame:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", 2, -2)
		elseif side == "top" then
			local split = PitBull:GetModule("Aura").db.profile[frame.group].split
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", -2, split and 18 or 2)
			frame.auraFrame:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 2, split and 18 or 2)
		elseif side == "left" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("BOTTOMRIGHT", frame, "LEFT", -2, 0)
		elseif side == "topright" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("TOPLEFT", frame, "TOPRIGHT", 2, 0)
		elseif side == "bottomright" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 2, 0)
		elseif side == "bottomleft" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", -2, 0)
		elseif side == "topleft" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("TOPRIGHT", frame, "TOPLEFT", -2, 0)
		elseif side == "right" then
			frame.auraFrame:SetWidth(160)
			frame.auraFrame:SetPoint("BOTTOMLEFT", frame, "RIGHT", 2, 0)
		end
	end
end
function ABF.auraFrame2(unit, frame)
	frame.auraFrame2:ClearAllPoints()
	local size = PitBull:GetModule("Aura").db.profile[frame.group].debuffsize
	local w = frame:GetWidth()
	frame.auraFrame2:SetHeight(size)
	frame.auraFrame2:SetWidth(w)
	local debuffside = ABF_db.profile[frame.group].debuffSide
	if debuffside == "bottom" then
		frame.auraFrame2:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", -2, -2)
	elseif debuffside == "top" then
		frame.auraFrame2:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", -2, 2)
	elseif debuffside == "left" then
		frame.auraFrame2:SetPoint("BOTTOMRIGHT", frame, "LEFT", w-size-2, -2)
	elseif debuffside == "right" then
		frame.auraFrame2:SetPoint("BOTTOMLEFT", frame, "RIGHT", 2, 0)
	elseif debuffside == "topleft" then
		frame.auraFrame2:SetPoint("TOPRIGHT", frame, "TOPLEFT", w-size-2, 0)
	elseif debuffside == "topright" then
		frame.auraFrame2:SetPoint("TOPLEFT", frame, "TOPRIGHT", 2, 0)
	elseif debuffside == "bottomleft" then
		frame.auraFrame2:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", w-size-2, 0)
	elseif debuffside == "bottomright" then
		frame.auraFrame2:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 2, 0)
	end
end
function ABF.portrait(unit, frame)
	local portraitSide = ABF_db.profile[frame.group].portraitSide
	local leftOrRight = portraitSide == "left" or portraitSide == "right"
	if portraitSide == "left" then
		frame.portrait:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
	elseif portraitSide == "right" then
		frame.portrait:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
	elseif portraitSide == "top" then
		frame.portrait:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		frame.portrait:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
	else -- bottom then
		frame.portrait:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
		frame.portrait:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
	end
	
	if leftOrRight then
		local size
		if frame.powerBar then
			size = figureHeight('powerBar', frame) + figureHeight('healthBar', frame)
		else
			size = figureHeight('healthBar', frame)
		end
		frame.portrait:SetWidth(size)
		frame.portrait:SetHeight(size)
	else
		local size = figureHeight('portrait', frame)
		frame.portrait:SetHeight(size)
	end
end
if playerClass == "ROGUE" or playerClass == "DRUID" then
	function ABF.comboPoints(unit, frame)
		if frame.group ~= "target" then return end 
		local side = ABF_db.profile[frame.group].comboSide
		if side == "left" or side == "right" then
			if side == "left" then
				frame.comboPoints:SetPoint( "RIGHT", frame, "LEFT", 0, 0)
			else
				frame.comboPoints:SetPoint( "LEFT", frame, "RIGHT", 0, 0)
			end
			frame.comboPoints:SetWidth( 10 )
			frame.comboPoints:SetHeight( 50 )
			for i = 1, 5 do
				local combo = frame.comboPoints["combo"..i]
				combo:ClearAllPoints()
				if i == 1 then
					combo:SetPoint("BOTTOM", frame.comboPoints, "BOTTOM")
				else
					combo:SetPoint("BOTTOM", frame.comboPoints["combo"..(i-1)], "TOP", 0, 2)
				end			
			end
		elseif side == "top" or side == "bottom" then
			if side == "top" then
				frame.comboPoints:SetPoint( "TOPRIGHT", frame, "TOPRIGHT", 0, 0)
			else
				frame.comboPoints:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
			end
			frame.comboPoints:SetWidth( 50 )
			frame.comboPoints:SetHeight( 10 )
			for i = 1, 5 do
				local combo = frame.comboPoints["combo"..i]
				combo:ClearAllPoints()
				if i == 1 then
					combo:SetPoint("RIGHT", frame.comboPoints, "RIGHT")
				else
					combo:SetPoint("RIGHT", frame.comboPoints["combo"..(i-1)], "LEFT")
				end	
			end
		end
	end
end
function ABF.combatIcon(unit, frame)
	frame.combatIcon:SetPoint("CENTER", frame, "BOTTOMLEFT", 0, 0)
	frame.combatIcon:SetWidth(24)
	frame.combatIcon:SetHeight(24)
end
function ABF.pvpIcon(unit, frame)
	frame.pvpIcon:SetPoint("CENTER", frame, "TOPRIGHT", 3, -5)
	frame.pvpIcon:SetWidth(27)
	frame.pvpIcon:SetHeight(27)
end
function ABF.restIcon(unit, frame)
	if frame.combatIcon then
		frame.restIcon:SetPoint("LEFT", frame.combatIcon, "RIGHT", 4, 0)
	else
		frame.restIcon:SetPoint("CENTER", frame, "BOTTOMLEFT", 0, 0)
	end
	frame.restIcon:SetWidth(24)
	frame.restIcon:SetHeight(24)
end
function ABF.leaderIcon(unit, frame)
	frame.leaderIcon:SetPoint("CENTER", frame, "TOPLEFT", 0, 0)
	frame.leaderIcon:SetWidth(18)
	frame.leaderIcon:SetHeight(18)
end
function ABF.masterIcon(unit, frame)
	if frame.leaderIcon then
		frame.masterIcon:SetPoint("LEFT", frame.leaderIcon, "RIGHT", 4, 0)
	else
		frame.masterIcon:SetPoint("CENTER", frame, "TOPLEFT", 0, 0)
	end
	frame.masterIcon:SetWidth(20)
	frame.masterIcon:SetHeight(20)
end

local function healthBarHeight_get(group)
	return ABF_db.profile[group].healthBarHeight
end
local function healthBarHeight_set(group, value)
	ABF_db.profile[group].healthBarHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.healthBar_options(group)
	return {
		name = L["Height"],
		desc = L["Relative height of the bar"],
		type = 'range',
		pass = true,
		passValue = group,
		get = healthBarHeight_get,
		set = healthBarHeight_set,
		min = 1,
		max = 12,
		step = 1,
	}
end

local function powerBarHeight_get(group)
	return ABF_db.profile[group].powerBarHeight
end
local function powerBarHeight_set(group, value)
	ABF_db.profile[group].powerBarHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.powerBar_options(group)
	return {
		name = L["Height"],
		desc = L["Relative height of the bar"],
		type = 'range',
		pass = true,
		passValue = group,
		get = powerBarHeight_get,
		set = powerBarHeight_set,
		min = 1,
		max = 12,
		step = 1,
	}
end

if playerClass == "DRUID" then
	local function druidManaBarHeight_get(group)
		return ABF_db.profile[group].druidManaBarHeight
	end
	local function druidManaBarHeight_set(group, value)
		ABF_db.profile[group].druidManaBarHeight = value
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			PitBull:UpdateLayout(frame)
		end
	end
	function ABF.druidManaBar_options(group)
		if group == "player" then
			return {
				name = L["Height Percentage"],
				desc = L["Percentage of height that the druid mana bar will take up of the normal power bar's height"],
				type = 'range',
				pass = true,
				passValue = group,
				get = druidManaBarHeight_get,
				set = druidManaBarHeight_set,
				min = 0.05,
				max = 0.95,
				step = 0.01,
				bigStep = 0.05,
				isPercent = true,
			}
		end
	end
end

local function castBarHeight_get(group)
	return ABF_db.profile[group].castBarHeight
end
local function castBarHeight_set(group, value)
	ABF_db.profile[group].castBarHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.castBar_options(group)
	return {
		name = L["Height"],
		desc = L["Relative height of the bar"],
		type = 'range',
		pass = true,
		passValue = group,
		get = castBarHeight_get,
		set = castBarHeight_set,
		min = 1,
		max = 12,
		step = 1,
	}
end

local function expBarHeight_get(group)
	return ABF_db.profile[group].expBarHeight
end
local function expBarHeight_set(group, value)
	ABF_db.profile[group].expBarHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.expBar_options(group)
	if group == 'player' or group == 'pet' then
		return {
			name = L["Height"],
			desc = L["Relative height of the bar"],
			type = 'range',
			pass = true,
			passValue = group,
			get = expBarHeight_get,
			set = expBarHeight_set,
			min = 1,
			max = 12,
			step = 1,
		}
	end
end

local function repBarHeight_get(group)
	return ABF_db.profile[group].repBarHeight
end
local function repBarHeight_set(group, value)
	ABF_db.profile[group].repBarHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.repBar_options(group)
	if group == 'player' then
		return {
			name = L["Height"],
			desc = L["Relative height of the bar"],
			type = 'range',
			pass = true,
			passValue = group,
			get = repBarHeight_get,
			set = repBarHeight_set,
			min = 1,
			max = 12,
			step = 1,
		}
	end
end

local function portraitSide_get(group)
	return ABF_db.profile[group].portraitSide
end
local function portraitSide_set(group, value)
	ABF_db.profile[group].portraitSide = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
local function portraitSide_isLeftOrRight(group)
	local portraitSide = ABF_db.profile[group].portraitSide
	return portraitSide == "left" or portraitSide == "right"
end
local function portraitHeight_get(group)
	return ABF_db.profile[group].portraitHeight
end
local function portraitHeight_set(group, value)
	ABF_db.profile[group].portraitHeight = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end
function ABF.portrait_options(group)
	return {
		name = L["Side"],
		desc = L["What side of the unit frame to position the portrait at."],
		type = "text",
		validate = {
			left = "Left",
			right = "Right",
			top = "Top",
		},
		get = portraitSide_get,
		set = portraitSide_set,
		passValue = group
	}, 	{
		name = L["Height"],
		desc = L["Relative height of the portrait"],
		type = 'range',
		pass = true,
		passValue = group,
		get = portraitHeight_get,
		set = portraitHeight_set,
		min = 1,
		max = 12,
		step = 1,
		disabled = portraitSide_isLeftOrRight,
	}
end

local function auraSide_get(group)
	return ABF_db.profile[group].auraSide
end
local function auraSide_set(group, value)
	ABF_db.profile[group].auraSide = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end

local function buffSide_get(group)
	return ABF_db.profile[group].buffSide
end
local function buffSide_set(group, value)
	ABF_db.profile[group].buffSide = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end

local function debuffSide_get(group)
	return ABF_db.profile[group].debuffSide
end
local function debuffSide_set(group, value)
	ABF_db.profile[group].debuffSide = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(frame)
	end
end

local function splitOptions_isHidden(group)
	local aura = PitBull:HasModule("Aura") and PitBull:GetModule("Aura")
	if not aura or not aura.db or not aura.db.profile or not aura.db.profile[group] then
		return true
	end
	return not aura.db.profile[group].split
end

local function nonsplitOptions_isHidden(group)
	local aura = PitBull:HasModule("Aura") and PitBull:GetModule("Aura")
	if not aura or not aura.db or not aura.db.profile or not aura.db.profile[group] then
		return true
	end
	return aura.db.profile[group].split
end

function ABF.auraFrame_options(group)
	return {
		name = L["Side"],
		desc = L["Which side to position all aura icons at."],
		type = "text",
		validate = {
			left = "Left",
			topleft = "Top-left",
			bottomleft = "Bottom-left",
			right = "Right",
			topright = "Top-right",
			bottomright = "Bottom-right",
			top = "Top",
			bottom = "Bottom",
		},
		get = auraSide_get,
		set = auraSide_set,
		passValue = group,
		hidden = function(group)
			for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
				if frame.auraFrame2 then
					return true
				end
			end	
			return false
		end,
		disabled = function(group)
			for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
				if frame.auraFrame then
					return false
				end
			end	
			return true
		end,
	}, {
		name = L["Side"],
		desc = L["Options to change the side which the auras are on."],
		type = "group",
		hidden = function()
			for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
				if frame.auraFrame2 then
					return false
				end
			end
			return true
		end,
		passValue = group,
		args = {
			buffside = {
				name = L["Buff side"],
				desc = L["Which side to position buff icons at."],
				type = "text",
				validate = {
					left = "Left",
					topleft = "Top-left",
					bottomleft = "Bottom-left",
					right = "Right",
					topright = "Top-right",
					bottomright = "Bottom-right",
					top = "Top",
					bottom = "Bottom",
				},
				get = buffSide_get,
				set = buffSide_set,
				passValue = group,
				disabled = splitOptions_isHidden,
			},
			debuffside = {
				name = L["Debuff side"],
				desc = L["Which side to position debuff icons at."],
				type = "text",
				validate = {
					left = "Left",
					topleft = "Top-left",
					bottomleft = "Bottom-left",
					right = "Right",
					topright = "Top-right",
					bottomright = "Bottom-right",
					top = "Top",
					bottom = "Bottom",
				},
				get = debuffSide_get,
				set = debuffSide_set,
				passValue = group,
				disabled = splitOptions_isHidden,
			},
		}
	}
end

if playerClass == "ROGUE" or playerClass == "DRUID" then
	local function comboSide_get(group)
		return ABF_db.profile[group].comboSide
	end
	local function comboSide_set(group, value)
		ABF_db.profile[group].comboSide = value
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			PitBull:UpdateLayout(frame)
		end
	end

	ABF.comboPoints_options_name = "Combo points"
	ABF.comboPoints_options_desc = "Options for the combo point display."
	function ABF.comboPoints_options(group)
		if group == "target" then
			return {
				name = L["Side"],
				desc = L["What side of the unit frame to position the combo point display at."],
				type = "text",
				validate = {
					left = "Left",
					right = "Right",
					top = "Top",
					bottom = "Bottom",
				},
				get = comboSide_get,
				set = comboSide_set,
				passValue = group,
			}
		end
	end
end

ABF.extraFrames = {}

local textStyles = {
	classText = {
		["Standard"]            = "[Classification] [Level:DifficultyColor] [SmartClass:ClassColor] [DruidForm:Paren] [SmartRace]",
		["Hide"]                = "",
		["Custom"]              = "Custom",
		["Player Classes Only"] = "[Classification] [Level:DifficultyColor] [PlayerClass:ClassColor] [DruidForm:Paren] [SmartRace]",
		["Short"]               = "[DifficultyColor][Level][Plus][White] [SmartRace]",
	},
	healthText = {
		["Absolute"]       = "[Status:SureHP:PercentHP:Percent]",
		["Absolute Short"] = "[Status:SureHP:Short:PercentHP:Percent]",
		["Difference"]     = "[Status:SmartMissingHP]",
		["Percent"]        = "[Status:PercentHP:Percent]",
		["Hide"]           = "",
		["Mini"]           = "[CurHP:VeryShort]",
		["Custom"]         = "Custom",
		["Smart"]          = "[IsFriend?Status:MissingHP:HideZero:Short:Color(ff7f7f)!Status:SmartHP:Short]",
		["Absolute and Percent"]  = "[~Status?FractionalHP:Short] || [Status:PercentHP:Percent]",
		["Informational"]  = "[IsFriend?~Status?MissingHP:HideZero:Short:Color(ff7f7f)] || [~Status?FractionalHP:Short] || [Status:PercentHP:Percent]"
	},
	nameText = {
		["Standard"]             = "[Name] [AFKDND:Angle]",
		["Hostility Colored"]    = "[Name:HostileColor] [AFKDND:Angle]",
		["Hide"]                 = "",
		["Custom"]               = "Custom",
		["Class Colored"]        = "[Name:ClassColor] [AFKDND:Angle]",
		["Long"]                 = "[Level] [Name:ClassColor] [AFKDND:Angle]",
		["Long w/ Druid form"]   = "[Level] [Name:ClassColor] [DruidForm:Paren] [AFKDND:Angle]",
	},
	powerText = {
		["Absolute"]       = "[~HasNoMP?FractionalMP]",
		["Absolute Short"] = "[~HasNoMP?FractionalMP:Short]",
		["Difference"]     = "[MissingMP:Negate]",
		["Percent"]        = "[PercentMP:Percent]",
		["Mini"]           = "[~HasNoMP?CurMP:VeryShort]",
		["Hide"]           = "",
		["Custom"]         = "Custom",
		["Smart"]          = "[MissingMP:HideZero:Short:Color(7f7fff)]",
	},
	comboText = {
		["Standard"]       = "[IsEnergy#player?Combos:HideZero]",
		["Hide"]           = "",
		["Custom"]         = "Custom",
	},
	experienceText = {
		["Standard"]       = "[FractionalXP] [PercentXP:Percent:Paren] [PercentRestXP:HideZero:Percent:Prepend(R: )]",
		["On Mouse-over"]       = "[IsMouseover?FractionalXP] [IsMouseover?PercentXP:Percent:Paren] [IsMouseover?PercentRestXP:HideZero:Percent:Prepend(R: )]",
		["Hide"]           = "",
		["Custom"]         = "Custom",
	},
	repText = {
		["Standard"]       = "[~IsMouseOver?FractionalRep] [~IsMouseOver?PercentRep:Percent:Paren!RepName]",
		["Hide"]           = "",
		["Custom"]         = "Custom",
	},
}

local function textExtraFrameCreation(key, func)
	assert(type(key) == "string")
	local styles = textStyles[key]
	assert(type(styles) == "table")
	return function(unit, frame)
		if func and not func(frame.group) then
			return
		end
		local text = newFrame("FontString", frame.overlay, "OVERLAY")
		frame[key] = text
		local font, fontsize = PitBull:GetFont()
		text:SetFont(font, fontsize * ABF_db.profile[frame.group][key .. "Size"])
		text:SetShadowColor(0, 0, 0, 1)
		text:SetShadowOffset(0.8, -0.8)
		text:SetNonSpaceWrap(false)
		text:SetJustifyH("LEFT")
		local code
		if ABF_db.profile[frame.group][key .. "Style"] == "Custom" then
			code = ABF_db.profile[frame.group][key .. "Custom"]
		else
			code = styles[ABF_db.profile[frame.group][key .. "Style"]]
		end
		DogTag:RegisterFontString(text, frame, unit, code or '')
	end
end

local getTextStyle = setmetatable({}, {__index=function(self, key)
	self[key] = function(group)
		return ABF_db.profile[group][key .. "Style"]
	end
	return self[key]
end})
local setTextStyle = setmetatable({}, {__index=function(self, key)
	self[key] = function(group, value)
		ABF_db.profile[group][key .. "Style"] = value

		local code
		if value == "Custom" then
			code = ABF_db.profile[group][key .. "Custom"]
		else
			code = textStyles[key][value]
			ABF_db.profile[group][key .. "Custom"] = code
		end
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			DogTag:RegisterFontString(frame[key], frame, unit, code)
		end
	end
	return self[key]
end})

local getTextCustom = setmetatable({}, {__index=function(self, key)
	self[key] = function(group)
		return ABF_db.profile[group][key .. "Custom"]
	end
	return self[key]
end})
local setTextCustom = setmetatable({}, {__index=function(self, key)
	self[key] = function(group, value)
		value = DogTag:FixCasing(value)
		ABF_db.profile[group][key .. "Custom"] = value

		if ABF_db.profile[group][key .. "Style"] ~= "Custom" then
			return
		end
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			DogTag:RegisterFontString(frame[key], frame, unit, value)
		end
	end
	return self[key]
end})

local getTextSize = setmetatable({}, {__index=function(self, key)
	self[key] = function(group)
		return ABF_db.profile[group][key .. "Size"]
	end
	return self[key]
end})
local setTextSize = setmetatable({}, {__index=function(self, key)
	self[key] = function(group, value)
		ABF_db.profile[group][key .. "Size"] = value

		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			PitBull:UpdateLayout(frame)
		end
	end
	return self[key]
end})

local basePositionValidate = {
	["healthBar-left"] = ("%s/%s"):format(L["Health bar"], L["Left"]),
	["healthBar-center"] = ("%s/%s"):format(L["Health bar"], L["Middle"]),
	["healthBar-right"] = ("%s/%s"):format(L["Health bar"], L["Right"]),
	["healthBar-outright"] = ("%s/%s"):format(L["Health bar"], L["Outside - Right"]),
	["healthBar-outleft"] = ("%s/%s"):format(L["Health bar"], L["Outside - Left"]),
	
	["powerBar-left"] = ("%s/%s"):format(L["Power bar"], L["Left"]),
	["powerBar-center"] = ("%s/%s"):format(L["Power bar"], L["Middle"]),
	["powerBar-right"] = ("%s/%s"):format(L["Power bar"], L["Right"]),
	["powerBar-outright"] = ("%s/%s"):format(L["Power bar"], L["Outside - Right"]),
	["powerBar-outleft"] = ("%s/%s"):format(L["Power bar"], L["Outside - Left"]),
	
	["expBar-left"] = ("%s/%s"):format(L["Experience bar"], L["Left"]),
	["expBar-center"] = ("%s/%s"):format(L["Experience bar"], L["Middle"]),
	["expBar-right"] = ("%s/%s"):format(L["Experience bar"], L["Right"]),
	["expBar-outright"] = ("%s/%s"):format(L["Experience bar"], L["Outside - Right"]),
	["expBar-outleft"] = ("%s/%s"):format(L["Experience bar"], L["Outside - Left"]),
	
	["repBar-left"] = ("%s/%s"):format(L["Reputation bar"], L["Left"]),
	["repBar-center"] = ("%s/%s"):format(L["Reputation bar"], L["Middle"]),
	["repBar-right"] = ("%s/%s"):format(L["Reputation bar"], L["Right"]),
	["repBar-outright"] = ("%s/%s"):format(L["Reputation bar"], L["Outside - Right"]),
	["repBar-outleft"] = ("%s/%s"):format(L["Reputation bar"], L["Outside - Left"]),
	
	["portrait-top"] = ("%s/%s"):format(L["Portrait"], L["Top"]),
	["portrait-center"] = ("%s/%s"):format(L["Portrait"], L["Middle"]),
	["portrait-bottom"] = ("%s/%s"):format(L["Portrait"], L["Bottom"]),
	
	["frame-topleft"] = ("%s/%s"):format(L["Frame"], L["Above-left"]),
	["frame-top"] = ("%s/%s"):format(L["Frame"], L["Above-middle"]),
	["frame-topright"] = ("%s/%s"):format(L["Frame"], L["Above-right"]),
	["frame-bottomleft"] = ("%s/%s"):format(L["Frame"], L["Below-left"]),
	["frame-bottom"] = ("%s/%s"):format(L["Frame"], L["Below-middle"]),
	["frame-bottomright"] = ("%s/%s"):format(L["Frame"], L["Below-right"]),
	["frame-lefttop"] = ("%s/%s"):format(L["Frame"], L["Left-top"]),
	["frame-left"] = ("%s/%s"):format(L["Frame"], L["Left-middle"]),
	["frame-leftbottom"] = ("%s/%s"):format(L["Frame"], L["Left-bottom"]),
	["frame-righttop"] = ("%s/%s"):format(L["Frame"], L["Right-top"]),
	["frame-right"] = ("%s/%s"):format(L["Frame"], L["Right-middle"]),
	["frame-rightbottom"] = ("%s/%s"):format(L["Frame"], L["Right-bottom"]),
	["frame-center"] = ("%s/%s"):format(L["Frame"], L["Center"]),
	
	["hide"] = L["Hidden"],
}

local setTextPosition = setmetatable({}, {__index=function(self, key)
	self[key] = function(group, value)
		if type(group) == "table" then
			group, value = unpack(group)
		end
		if value ~= "hide" then
			local frame = findFrameAtPosition(value)
			if frame then
				ABF_db.profile[group][frame] = ABF_db.profile[group][key .. "Position"]
			end
		end
		ABF_db.profile[group][key .. "Position"] = value
	
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			PitBull:UpdateLayout(frame)
		end
	end
	return self[key]
end})
local getTextPosition = setmetatable({}, {__index=function(self, key)
	self[key] = function(group)
		if type(group) == "table" then
			local value
			group, value = unpack(group)
			return ABF_db.profile[group][key .. "Position"] == value
		end
		return ABF_db.profile[group][key .. "Position"]
	end
	return self[key]
end})

local mt = {__index=function(self, key)
	local t = {}
	
	for k, v in pairs(basePositionValidate) do
		local attach, point = ('-'):split(k)
		local attachName, pointName = ('/'):split(v)
		if point then
			if not t[attach] then
				t[attach] = {
					type = 'text',
					name = attachName,
					desc = attachName,
					validate = {},
					get = getTextPosition[self.key],
					set = setTextPosition[self.key],
					passValue = key,
					hidden = attach ~= "frame" and function(key)
						for unit, frame in PitBull:IterateUnitFramesByGroup(key) do
							if frame[attach] then
								return false
							end
						end
						return true
					end
				}
			end
			t[attach].validate[k] = pointName
		else
			t[attach] = {
				type = 'toggle',
				name = v,
				desc = v,
				get = getTextPosition[self.key],
				set = setTextPosition[self.key],
				passValue = { key, k },
			}
		end
	end
	self[key] = t
	return t
end}
local positionArgs = setmetatable({}, {__index=function(self, key)
	local t = setmetatable({key=key}, mt)
	self[key] = t
	return t
end})

local styleToValidate = setmetatable({}, {__index=function(self, key)
	local t = {}
	for k in pairs(key) do
		t[k] = k
	end
	self[key] = t
	return t
end})

local function textOptions(key, noStyle)
	assert(type(key) == "string")
	local styles = textStyles[key]
	assert(type(styles) == "table" or noStyle)
	if not noStyle then
		return function(group)
			return {
				type = "text",
				name = L["Style"],
				desc = L["Use a preconfigured style."],
				validate = styleToValidate[styles],
				validateDesc = styles,
				get = getTextStyle[key],
				set = setTextStyle[key],
				passValue = group,
			}, {
				type = "text",
				name = L["Style - Custom"],
				desc = L["Create a custom style for this unit type."],
				get = getTextCustom[key],
				set = setTextCustom[key],
				usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
				passValue = group,
			}, {
				type = "group",
				name = L["Position"],
				desc = L["Position of the text."],
				args = positionArgs[key][group],
			}, {
				type = 'range',
				name = L["Size"],
				desc = L["Font size of the text."],
				get = getTextSize[key],
				set = setTextSize[key],
				passValue = group,
				min = 0.25,
				max = 3,
				step = 0.01,
				bigStep = 0.05,
				isPercent = true,
			}
		end
	else
		return function(group)
			return {
				type = "group",
				name = L["Position"],
				desc = L["Position of the text."],
				args = positionArgs[key][group],
			}
		end
	end
end

ABF.combatText_options_name = "Combat text"
ABF.combatText_options_desc = "Customize the combat text."
ABF.combatText_options = textOptions('combatText', true)

ABF.extraFrames.classText = textExtraFrameCreation('classText')
ABF.classText_options_name = "Class text"
ABF.classText_options_desc = "Customize the class text."
ABF.classText_options = textOptions('classText')

ABF.extraFrames.healthText = textExtraFrameCreation('healthText')
ABF.healthText_options_name = "Health text"
ABF.healthText_options_desc = "Customize the health text."
ABF.healthText_options = textOptions('healthText')

ABF.extraFrames.nameText = textExtraFrameCreation('nameText')
ABF.nameText_options_name = "Name text"
ABF.nameText_options_desc = "Customize the name text."
ABF.nameText_options = textOptions('nameText')

ABF.extraFrames.powerText = textExtraFrameCreation('powerText')
ABF.powerText_options_name = "Power text"
ABF.powerText_options_desc = "Customize the power text."
ABF.powerText_options = textOptions('powerText')

if playerClass == "ROGUE" or playerClass == "DRUID" then
	ABF.extraFrames.comboText = textExtraFrameCreation('comboText', function(group) return group == "target" end)
	ABF.comboText_options_name = "Combo text"
	ABF.comboText_options_desc = "Customize the combo points text."
	ABF.comboText_options = textOptions('comboText')
end

local hasPetExp = playerClass == "HUNTER"
ABF.extraFrames.experienceText = textExtraFrameCreation('experienceText', function(group) return group == "player" or (hasPetExp and group == "pet") end)
ABF.experienceText_options_name = "Experience text"
ABF.experienceText_options_desc = "Customize the experience text."
ABF.experienceText_options = textOptions('experienceText')

ABF.extraFrames.repText = textExtraFrameCreation('repText', function(group) return group == "player" end)
ABF.repText_options_name = "Reputation text"
ABF.repText_options_desc = "Customize the reputation text."
ABF.repText_options = textOptions('repText')

PitBull:RegisterLayout("ABF", ABF)
